@@ -85,10 +85,24 @@ class RosRpmGenerator(RpmGenerator):
             self.rpm_inc,
             [p.name for p in self.packages.values()],
             releaser_history=releaser_history,
-            fallback_resolver=fallback_resolver
+            fallback_resolver=fallback_resolver,
+            skip_keys=self.skip_keys
         )
         subs['Package'] = rosify_package_name(subs['Package'], self.rosdistro)
 
+        # Virtual packages
+        subs['Provides'] += [
+            '%%{name}-%s = %%{version}-%%{release}' % subpackage for subpackage in [
+                'devel', 'doc', 'runtime']]
+
+        # Group membership
+        subs['Provides'].extend(
+            sanitize_package_name(rosify_package_name(g.name, self.rosdistro)) +
+            '(member)' for g in package.member_of_groups)
+        subs['Supplements'].extend(
+            sanitize_package_name(rosify_package_name(g.name, self.rosdistro)) +
+            '(all)' for g in package.member_of_groups)
+
         # ROS 2 specific bloom extensions.
         ros2_distros = [
             name for name, values in get_index().distributions.items()
@@ -107,15 +121,22 @@ class RosRpmGenerator(RpmGenerator):
                     self.rosdistro not in ('r2b2', 'r2b3', 'ardent') and \
                     'rosidl_interface_packages' in [p.name for p in package.member_of_groups]:
                 ROS2_VENDOR_TYPESUPPORT_DEPENDENCIES = [
-                    'rmw-connext-cpp',
-                    'rmw-fastrtps-cpp',
-                    'rmw-implementation',
-                    'rmw-opensplice-cpp',
                     'rosidl-typesupport-connext-c',
                     'rosidl-typesupport-connext-cpp',
-                    'rosidl-typesupport-opensplice-c',
-                    'rosidl-typesupport-opensplice-cpp',
+                    'rosidl-typesupport-fastrtps-c',
+                    'rosidl-typesupport-fastrtps-cpp',
                 ]
+                # OpenSplice was dropped after Eloquent.
+                # rmw implementations are required as dependencies up to Eloquent.
+                if self.rosdistro in ['bouncy', 'crystal', 'dashing', 'eloquent']:
+                    ROS2_VENDOR_TYPESUPPORT_DEPENDENCIES.extend([
+                        'rmw-connext-cpp',
+                        'rmw-fastrtps-cpp',
+                        'rmw-implementation',
+                        'rmw-opensplice-cpp',
+                        'rosidl-typesupport-opensplice-c',
+                        'rosidl-typesupport-opensplice-cpp',
+                    ])
 
                 subs['BuildDepends'] += [
                     rosify_package_name(name, self.rosdistro) for name in ROS2_VENDOR_TYPESUPPORT_DEPENDENCIES]
